#include "UpperTesterDenmFactory.hh"

#include "UpperTesterTypes.hh"

#include "loggers.hh"

UpperTesterDenmLayer::UpperTesterDenmLayer(const std::string & p_type, const std::string & param) : t_layer<LibItsDenm__TestSystem::UpperTesterPort>(p_type), _params(), _codec() {
  loggers::get_instance().log(">>> UpperTesterDenmLayer::UpperTesterDenmLayer: %s, %s", to_string().c_str(), param.c_str());
  // Setup parameters
  Params::convert(_params, param);
  Params::const_iterator it = _params.find(Params::loopback);
  if (it == _params.cend()) {
    _params.insert(std::pair<std::string, std::string>(Params::loopback, "0"));
  }
  it = _params.find(Params::station_id);
  if (it == _params.cend()) {
    _params.insert(std::pair<std::string, std::string>(Params::station_id, "12345"));
  }
}

void UpperTesterDenmLayer::sendMsg(const LibItsDenm__TypesAndValues::UtDenmInitialize& send_par, Params& params){
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::sendMsg", send_par);

  if (_params[Params::loopback].compare("1") == 0) {
    LibItsDenm__TypesAndValues::UtDenmResults ut_denm_results;
    ut_denm_results.utDenmInitializeResult() = BOOLEAN(true);
    to_all_upper_ports(ut_denm_results, params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode((const Record_Type &)send_par, data);
    // Update parameters
    send_data(data, params);
  }
}

void UpperTesterDenmLayer::sendMsg(const LibItsDenm__TypesAndValues::UtDenmChangePosition& send_par, Params& params)
{
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::sendMsg", send_par);

  if (_params[Params::loopback].compare("1") == 0) {
    LibItsDenm__TypesAndValues::UtDenmResults ut_denm_results;
    ut_denm_results.utDenmChangePositionResult() = BOOLEAN(true);
    to_all_upper_ports(ut_denm_results, params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode((const Record_Type &)send_par, data);
    // Update parameters
    send_data(data, params);
  }
}

void UpperTesterDenmLayer::sendMsg(const LibItsDenm__TypesAndValues::UtDenmChangePseudonym& send_par, Params& params)
{
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::sendMsg", send_par);

  if (_params[Params::loopback].compare("1") == 0) {
    LibItsDenm__TypesAndValues::UtDenmResults ut_denm_results;
    ut_denm_results.utDenmChangePseudonymResult() = BOOLEAN(true);
    to_all_upper_ports(ut_denm_results, params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode((const Record_Type &)send_par, data);
    // Update parameters
    send_data(data, params);
  }
}

void UpperTesterDenmLayer::sendMsg(const LibItsDenm__TypesAndValues::UtDenmTrigger& send_par, Params& params)
{
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::sendMsg", send_par);

  if (_params[Params::loopback].compare("1") == 0) {
    LibItsDenm__TypesAndValues::UtDenmTriggerResult trigger_result(true, ITS__Container::ActionID(std::stoi(_params[Params::station_id]), 1));
    LibItsDenm__TypesAndValues::UtDenmResults ut_denm_results;
    ut_denm_results.utDenmTriggerResult() = trigger_result;
    to_all_upper_ports(ut_denm_results, params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode((const Record_Type &)send_par, data);
    // Update parameters
    send_data(data, params);
  }
}


void UpperTesterDenmLayer::sendMsg(const LibItsDenm__TypesAndValues::UtDenmUpdate& send_par, Params& params)
{
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::sendMsg", send_par);

  if (_params[Params::loopback].compare("1") == 0) {
    LibItsDenm__TypesAndValues::UtDenmUpdateResult update_result(true, ITS__Container::ActionID(std::stoi(_params[Params::station_id]), 1));
    LibItsDenm__TypesAndValues::UtDenmResults ut_denm_results;
    ut_denm_results.utDenmUpdateResult() = update_result;
    to_all_upper_ports(ut_denm_results, params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode((const Record_Type &)send_par, data);
    // Update parameters
    send_data(data, params);
  }
}

void UpperTesterDenmLayer::sendMsg(const LibItsDenm__TypesAndValues::UtDenmTermination& send_par, Params& params)
{
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::sendMsg", send_par);

  if (_params[Params::loopback].compare("1") == 0) {
    LibItsDenm__TypesAndValues::UtDenmResults ut_denm_results;
    ut_denm_results.utDenmTerminationResult() = BOOLEAN(true);
    to_all_upper_ports(ut_denm_results, params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode((const Record_Type &)send_par, data);
    // Update parameters
    send_data(data, params);
  }
}

void UpperTesterDenmLayer::send_data(OCTETSTRING& data, Params& params) {
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::send_data: ", data);
  //params.log();
  send_to_all_layers(data, params);
}

void UpperTesterDenmLayer::receive_data(OCTETSTRING& data, Params& params)
{
  loggers::get_instance().log_msg(">>> UpperTesterDenmLayer::receive_data: ", data);
 
  std::unique_ptr<Base_Type> r = _codec.decode(data);
  if (r.get() != nullptr) {
    // Pass it to the ports if amy
    to_all_upper_ports(*r, params);
  }
}

UpperTesterDenmFactory UpperTesterDenmFactory::_f;
